#ifndef COMMON_H
#define COMMON_H

#include <syslog.h>
#include <unistd.h>
#include <utmp.h>
#include "types.h"
#include "macros.h"
#include "list.h"

#undef DEBUG

#define FRAME_CHAR		0xff

#define MAX_CHANNELS		256
#define MAX_CLIENTS		256
#define MAX_NETNODES		64
#define MAX_CMDLEN		1024
#define MAX_LINES            	4
#define MAX_LINELEN		(COLS + 1)

#define CMD_PING		0x0000
#define CMD_SERVERINFO		0x0001
#define CMD_CLIENTINFO		0x0002
#define CMD_NICKNAME		0x0003
#define CMD_GETCHANNELNO	0x0004
#define CMD_LISTCHANNELS	0x0005
#define CMD_LISTCLIENTS		0x0006
#define CMD_JOINCHANNEL		0x0007
#define CMD_LEAVECHANNEL	0x0008
#define CMD_ACTION		0x0009
#define CMD_AUTOMSG		0x000a
#define CMD_AUTODEL		0x000b
#define CMD_GETAUTOMSGS		0x000c
#define CMD_MSGUSER		0x000d
#define CMD_MSGALL		0x000e
#define CMD_CONNECT		0x000f
#define CMD_DISCONNECT		0x0010

#define MSG(x)			((x) & 0x00ff)
#define RES(x)			((x) & 0xff00)

#define MSG_PING		0x0001
#define MSG_NICKNAME		0x0002
#define MSG_CONNECT		0x0003
#define MSG_DISCONNECT 		0x0004
#define MSG_CONNREQ		0x0005
#define MSG_CLIENTJOINED       	0x0006
#define MSG_CLIENTLEFT		0x0007
#define MSG_CLIENTCONN		0x0008
#define MSG_CLIENTDISC		0x0009
#define MSG_SIGNON		0x000a
#define MSG_SIGNOFF		0x000b
#define MSG_ACTION		0x000c
#define MSG_AUTOMSG		0x000d
#define MSG_DONEAUTOMSGS	0x000e
#define MSG_BROADCAST		0x000f
#define MSG_MSGSENT		0x0010
#define MSG_PRIVMSG		0x0011

#define RES_OK			0x0100
#define RES_EILLEGAL		0x0200
#define RES_EBADNICK		0x0300
#define RES_ENICKINUSE		0x0400
#define RES_ENOUSER      	0x0500
#define RES_ENOCHANNEL		0x0600
#define RES_ENOTLOCAL		0x0700
#define RES_EBUSY		0x0800
#define RES_ENETBUSY		0x0900
#define RES_ENETFULL		0x0a00

typedef struct ServerInfo {
  char                     name[64];
  time_t                   startTime;
  time_t                   upTime;
  int                      maxNet;
  int                      nChannels;
  int                      nClients;
  int                      netConnections;
  int                      activeNetConnections;
  int                      activeGames;
} ServerInfo;

typedef struct Msg {
  word                     type,
                           len;
  char                    *data;
} Msg;

typedef struct AutoMsg {
  char                     toName[13],
                           fromName[13];
  char                     msg[MAX_CMDLEN];
  byte                     seen[MAX_CLIENTS / 8];
  time_t                   expire;
} AutoMsg;

typedef struct Channel {
  char                     name[13],
                           topic[64];
  int                      nConnects;
  int                      nClients;
  int                      maxNet;
  int                      gameNo;
  int                      dynamic;
  int                      nAutoMsgs;
  List                     autoMsg;
} Channel;

typedef struct Game {
  int                      clientNo;
  char                     setup[256];
} Game;

typedef struct {
  int                      fd;
  int                      channel;
  int                      busy;
  int                      ccid,
                           nodeAddr;
  time_t                   lastPingSent,
                           lastPingRcvd;
  char                     name[13],
                           gecos[32],
                           line[UT_LINESIZE];
} Client;

int                      getToken(char *d, char *s, int *idx);
void                     errQuit(char *fmt,...);
void                     perrQuit(char *name);

#endif
