#include <assert.h>
#include <malloc.h>
#include <string.h>
#include "x_err.hpp"
#include "x_huff.hpp"

#define LEFT(a)		left[a - MAX_CHAR]
#define RIGHT(a)	right[a - MAX_CHAR]

short Huff::build_tree() {
  short root, i, min_1, min_2;
  freq[TWICE_MAX - 1] = 0xffff;
  for (root = MAX_CHAR; ; root++) {
    min_1 = min_2 = TWICE_MAX - 1;
    for (i = 0; i < root; i++) {
      if (freq[i]) {
	if (freq[i] < freq[min_1]) {
	  min_2 = min_1;
	  min_1 = i;
	} else if (freq[i] < freq[min_2]) min_2 = i;
      }
    }
    if (min_2 == TWICE_MAX - 1) break;
    freq[root] = freq[min_1] + freq[min_2];
    LEFT(root) = min_1; RIGHT(root) = min_2;
    freq[min_1] = freq[min_2] = 0;
  }
  return root - 1;
}

void Huff::gen_codes(short node, ushort code, byte code_len) {
  if (node < MAX_CHAR) {
    Huff::code[node] = code;
    Huff::code_len[node] = code_len;
  } else {
    code <<= 1;
    code_len++;
    gen_codes(LEFT(node), code, code_len);
    gen_codes(RIGHT(node), code | 1, code_len);
  }
}

void Huff::Encode_Symbol(int c, ushort *code, int *code_len) {
  *code = Huff::code[c];
  *code_len = Huff::code_len[c];
}

int Huff::Decode_Symbol(byte *mask, byte **data) {
  int a = root;
  do {
    if (**data & *mask) a = RIGHT(a);
    else a = LEFT(a);
    *mask >>= 1;
    if (!*mask) {
      *mask = 0x80;
      (*data)++;
    }
  } while (a >= MAX_CHAR);
  return a;
}

Huff::Huff(FILE *f) {
  ushort i;
  byte c;
  left = (short __far*)_fmalloc(MAX_CHAR * sizeof(short));
  right = (short __far*)_fmalloc(MAX_CHAR * sizeof(short));
  code = (ushort __far*)_fmalloc(MAX_CHAR * sizeof(ushort));
  code_len = (byte __far*)_fmalloc(MAX_CHAR * sizeof(byte));
  freq = (ushort __far*)_fmalloc(TWICE_MAX * sizeof(ushort));
  if (!left || !right || !code || !code_len || !freq) fatalError("memory allocation failed");
  for (i = 0; i < MAX_CHAR; i++) {
    fread((void*)&c, sizeof(byte), 1, f);
    freq[i] = c;
  }
  root = build_tree(); _ffree(freq);
  gen_codes(root, 0, 0);
}

Huff::~Huff() {
  _ffree(left);
  _ffree(right);
  _ffree(code);
  _ffree(code_len);
}
